<?xml version="1.0" encoding="UTF-8"?>
<!-- 
License: MIT

Copyright 2019 Frederik Elwert <frederik.elwert@rub.de>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:tei="http://www.tei-c.org/ns/1.0" xmlns:exsl="http://exslt.org/common"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  exclude-result-prefixes="tei exsl xsl xlink">

  <xsl:output method="html" encoding="utf-8" indent="yes"
    doctype-system="about:legacy-compat"/>
  <xsl:variable name="root" select="/"/>
  <!-- Parse related documents -->
  <xsl:variable name="related-items" select="/tei:TEI/tei:teiHeader/tei:fileDesc/tei:notesStmt"/>
  <xsl:variable name="translation-doc" select="document($related-items/tei:relatedItem[@type='translation']/tei:ptr/@target)"/>
  <xsl:variable name="introduction-doc" select="document($related-items/tei:relatedItem[@type='introduction']/tei:ptr/@target)"/>
  <xsl:variable name="glossary-doc" select="document($related-items/tei:relatedItem[@type='glossary']/tei:ptr/@target)"/>
  <xsl:param name="filename"/>
  <xsl:variable name="thumbnails" select="true()" />
  <xsl:variable name="thumbnail_suffix">_thumb.jpg</xsl:variable>

  <xsl:key name="wit" match="tei:witness" use="@xml:id"/>
  <xsl:key name="gloss" match="tei:note" use="@target"/>
  <xsl:key name="trans" match="tei:ab" use="@corresp"/>
  <xsl:key name="hand" match="tei:handNote" use="@xml:id"/>
  <xsl:key name="term" match="tei:gloss" use="@xml:id"/>
  <xsl:key name="bib" match="tei:bibl" use="@xml:id"/>
  <xsl:key name="ref" match="tei:bibl" use="@corresp"/>

  <xsl:variable name="rtl-languages">
    <language>ar</language>
    <language>he</language>
  </xsl:variable>

  <xsl:variable name="paper-types">
    <type>journalArticle</type>
    <type>bookSection</type>
    <type>encyclopediaArticle</type>
    <type>presentation</type>
  </xsl:variable>

  <!-- TEMPLATES -->

  <!-- TEMPLATE: modal

  Used to create modal dialogs for Bootstrap.

  -->

  <xsl:template name="modal">
    <xsl:param name="id"/>
    <xsl:param name="title"/>
    <xsl:param name="content"/>
    <xsl:variable name="source-url" select="substring-before(@target, '#')"/>
    <xsl:variable name="source-fragment" select="substring-after(@target, '#')"/>
    <xsl:variable name="source-doc" select="document($source-url)"/>
    <xsl:variable name="source-node" select="$source-doc//*[@xml:id = $source-fragment]"/>
    <div class="modal fade" id="{ $id }" tabindex="-1" role="dialog"
      aria-labelledby="title-{ generate-id($content) }">
      <div class="modal-dialog modal-lg" role="document">
        <div class="modal-content">
          <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-label="Close">
              <span aria-hidden="true">×</span>
            </button>
            <h4 class="modal-title" id="title-{ generate-id($content) }">
              <xsl:value-of select="$title"/>
            </h4>
          </div>
          <div class="modal-body">
            <xsl:if test="ancestor-or-self::tei:*/@xml:lang">
              <xsl:attribute name="lang">
                <xsl:value-of select="(ancestor-or-self::tei:*/@xml:lang)[1]"/>
              </xsl:attribute>
            </xsl:if>
            <xsl:if test="(ancestor-or-self::tei:*/@xml:lang)[1] = exsl:node-set($rtl-languages)/*">
              <xsl:attribute name="dir">rtl</xsl:attribute>
            </xsl:if>
            <xsl:copy-of select="$content"/>
          </div>
          <div class="modal-footer">
            <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
          </div>
        </div>
      </div>
    </div>
  </xsl:template>

  <!-- TEMPLATE: popup

  Used to create a popup attached to a button.

  -->

  <xsl:template name="popup">
    <xsl:param name="label"/>
    <xsl:param name="icon">comment</xsl:param>
    <xsl:param name="title"/>
    <xsl:param name="content"/>
    <xsl:param name="placement"/>
    <span dir="ltr">
      <button type="button" class="btn btn-default btn-xs" data-toggle="popover"
        data-trigger="focus" data-content="{.}" title="{$title}">
        <xsl:if test="$placement">
          <xsl:attribute name="data-placement">
            <xsl:value-of select="$placement"/>
          </xsl:attribute>
        </xsl:if>
        <xsl:attribute name="data-content">
          <xsl:value-of select="$content"/>
        </xsl:attribute>
        <xsl:choose>
          <xsl:when test="$label">
            <xsl:value-of select="$label"/>
          </xsl:when>
          <xsl:otherwise>
            <span class="glyphicon glyphicon-{$icon}"/>
          </xsl:otherwise>
        </xsl:choose>
      </button>
    </span>
  </xsl:template>

  <!-- TEMPLATE: popover

  Used to create a popover that appears on hovering over an element.

  -->

  <xsl:template name="popover">
    <xsl:param name="title"/>
    <xsl:param name="content"/>
    <xsl:param name="placement"/>
    <xsl:param name="class" />
    <xsl:param name="text"/>
    <span class="text-success" data-toggle="popover" data-trigger="hover" title="{$title}">
      <xsl:if test="normalize-space($class)">
        <xsl:attribute name="class">
          <xsl:value-of select="$class" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="normalize-space($placement)">
        <xsl:attribute name="data-placement">
          <xsl:value-of select="$placement"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:attribute name="data-content">
        <xsl:value-of select="$content"/>
      </xsl:attribute>
      <xsl:copy-of select="$text"/>
    </span>
  </xsl:template>

  <!-- MODES -->

  <!-- MODE: modal

  Used to create modal dialogs for specific elements. Uses the modal template.

  -->

  <xsl:template match="tei:ab" mode="modal">
    <xsl:variable name="trans-fragment" select="substring-after(@corresp, '#')"/>
    <xsl:variable name="content">
      <xsl:apply-templates/>
      <xsl:if test=".//tei:note[@type = 'footnote']">
        <section class="footnotes">
          <h1>Notes</h1>
          <xsl:apply-templates select=".//tei:note[@type = 'footnote']" mode="footnote" />
        </section>
      </xsl:if>
    </xsl:variable>
    <xsl:call-template name="modal">
      <xsl:with-param name="id"
        select="concat('trans-modal-', translate($trans-fragment, '#.', '--'))"/>
      <xsl:with-param name="title">Translation</xsl:with-param>
      <xsl:with-param name="content" select="exsl:node-set($content)"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="tei:pb" mode="modal">
    <xsl:variable name="thumb_img">
      <xsl:choose>
        <xsl:when test="not($thumbnails)"><xsl:value-of select="@facs" /></xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat(substring-before(@facs, '.'), $thumbnail_suffix)" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="content">
      <div style="text-align: center;">
        <svg xmlns="http://www.w3.org/2000/svg" width="500" height="400">
          <a xlink:href="{@facs}" target="_blank">
              <image xlink:href="{$thumb_img}" width="100%" height="100%"/>
          </a>
        </svg>
      </div>
    </xsl:variable>
    <xsl:call-template name="modal">
      <xsl:with-param name="id" select="concat('facs-modal-', generate-id())"/>
      <xsl:with-param name="title">Facsimile</xsl:with-param>
      <xsl:with-param name="content" select="exsl:node-set($content)"/>
    </xsl:call-template>
  </xsl:template>

  <!-- MODE: serialize

  Used to convert XML elements into a string representation.

  -->

  <xsl:template match="*" mode="serialize">
    <xsl:text>&lt;</xsl:text>
    <xsl:value-of select="name()"/>
    <xsl:apply-templates select="@*" mode="serialize"/>
    <xsl:choose>
      <xsl:when test="node()">
        <xsl:text>&gt;</xsl:text>
        <xsl:apply-templates mode="serialize"/>
        <xsl:text>&lt;/</xsl:text>
        <xsl:value-of select="name()"/>
        <xsl:text>&gt;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text> /&gt;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@*" mode="serialize">
    <xsl:text> </xsl:text>
    <xsl:value-of select="name()"/>
    <xsl:text>="</xsl:text>
    <xsl:value-of select="."/>
    <xsl:text>"</xsl:text>
  </xsl:template>

  <xsl:template match="text()" mode="serialize">
    <xsl:value-of select="."/>
  </xsl:template>

  <!-- MODE: panel

  Used to include divs from the introduction in a panel layout

  -->

  <xsl:template match="tei:div" mode="panel">
    <xsl:variable name="panel-id" select="@xml:id"/>
    <div id="{@xml:id}" class="panel panel-meta">
      <div class="panel-heading">
        <xsl:apply-templates select="tei:head" mode="panel"/>
      </div>
      <div id="panel-{$panel-id}" class="panel-collapse collapse">
        <div class="panel-body">
          <xsl:apply-templates select="*[not(self::tei:head)]"/>
          <xsl:if test=".//tei:note[@type = 'footnote']">
            <section class="footnotes">
              <h1>Notes</h1>
              <xsl:apply-templates select=".//tei:note[@type = 'footnote']" mode="footnote" />
            </section>
          </xsl:if>
        </div>
      </div>
    </div>
  </xsl:template>

  <xsl:template match="tei:head" mode="panel">
    <xsl:variable name="panel-id" select="parent::tei:div/@xml:id"/>
    <h2 class="panel-title">
      <a role="button" href="#panel-{$panel-id}" data-toggle="collapse"><xsl:apply-templates /></a>
    </h2>
  </xsl:template>

  <!-- Base Template -->

  <xsl:template match="/tei:TEI">
    <html>
      <head>
        <meta charset="utf-8"/>
        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>
        <meta name="viewport" content="width=device-width, initial-scale=1"/>
        <title>
          <xsl:value-of select="tei:teiHeader/tei:fileDesc/tei:titleStmt/tei:title"/>
        </title>
        <!-- Bootstrap CSS -->
        <link rel="stylesheet" href="css/bootstrap.min.css"/>
        <link rel="stylesheet" href="css/custom.css"/>
      </head>
      <body>
        <header>
          <xsl:variable name="header" select="tei:teiHeader/tei:fileDesc/tei:titleStmt"/>
          <xsl:if test="$header/tei:title">
            <h1>
              <xsl:if test="$header/tei:title/@xml:lang">
                <xsl:attribute name="lang">
                  <xsl:value-of select="$header/tei:title/@xml:lang"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:if test="$header/tei:title/@xml:lang = exsl:node-set($rtl-languages)/*">
                <xsl:attribute name="dir">rtl</xsl:attribute>
              </xsl:if>
              <xsl:value-of select="$header/tei:title"/>
            </h1>
            <xsl:if test="$header/tei:author">
              <h1>
                <small>
                  <xsl:text> by </xsl:text>
                  <xsl:value-of select="$header/tei:author"/>
                </small>
              </h1>
            </xsl:if>
          </xsl:if>
          <div class="lead">
            <xsl:variable name="editor" select="$header/tei:editor[not(@role)]"/>
            <xsl:variable name="translator" select="$header/tei:editor[@role = 'translator']"/>
            <xsl:choose>
              <xsl:when test="$editor and $translator and $editor = $translator">
                <p>
                  <xsl:text>Edition and Translation: </xsl:text>
                  <xsl:value-of select="$editor" />
                </p>
              </xsl:when>
              <xsl:otherwise>
                <xsl:if test="$editor">
                  <p>
                    <xsl:text>Edition: </xsl:text>
                    <xsl:value-of select="$editor"/>
                  </p>
                </xsl:if>
                <xsl:if test="$translator">
                  <p>
                    <xsl:text>Translation: </xsl:text>
                    <xsl:value-of select="$translator"/>
                  </p>
                </xsl:if>
              </xsl:otherwise>
            </xsl:choose>
            <xsl:for-each select="tei:teiHeader/tei:fileDesc/tei:editionStmt/tei:respStmt">
              <p>
                <xsl:value-of select="tei:resp"/>
                <xsl:text>: </xsl:text>
                <xsl:for-each select="tei:persName">
                  <xsl:value-of select="." />
                  <xsl:if test="position() != last()">
                    <xsl:text> and </xsl:text>
                  </xsl:if>
                </xsl:for-each>
              </p>
            </xsl:for-each>
          </div>
        </header>
        <aside class="panel-group">
          <xsl:if test="tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit">
            <div id="witnesses" class="panel panel-meta">
              <div class="panel-heading">
                <h2 class="panel-title"><a role="button" href="#panel-witnesses" data-toggle="collapse">Manuscript Witnesses</a></h2>
              </div>
              <div id="panel-witnesses" class="panel-collapse collapse">
                <div class="panel-body">
                  <ul>
                    <xsl:for-each
                      select="tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit/tei:witness">
                      <li><xsl:value-of select="@n"/>: MS <xsl:value-of
                          select="normalize-space(tei:msDesc/tei:msIdentifier)"/>
                        <xsl:if test="tei:msDesc/tei:msContents/tei:msItemStruct/tei:locus">
                          <xsl:text>, </xsl:text>
                          <xsl:value-of select="tei:msDesc/tei:msContents/tei:msItemStruct/tei:locus"/>
                        </xsl:if>
                        <xsl:if test="tei:msDesc/tei:history/tei:origin">
                          <xsl:text>, Copy Date: </xsl:text>
                          <xsl:value-of select="tei:msDesc/tei:history/tei:origin"/>
                          <xsl:if test="tei:msDesc/tei:history/tei:origin/@when">
                            <xsl:text> (</xsl:text>
                            <xsl:value-of select="tei:msDesc/tei:history/tei:origin/@when"/>
                            <xsl:text>)</xsl:text>
                          </xsl:if>
                        </xsl:if>
                      </li>
                    </xsl:for-each>
                  </ul>
                </div>
              </div>
            </div>
          </xsl:if>
          <xsl:apply-templates select="($introduction-doc|$glossary-doc)/tei:TEI/tei:text/tei:body/tei:div" mode="panel"/>
          <div id="help" class="panel panel-meta">
            <div class="panel-heading">
              <h2 class="panel-title"><a role="button" href="#panel-help" data-toggle="collapse"><span class="glyphicon glyphicon-question-sign" aria-hidden="true"></span> Navigating the Edition</a></h2>
            </div>
            <div id="panel-help" class="panel-collapse collapse">
              <div class="panel-body">
                <h3>Translations</h3>
                <ul>
                  <li>clicking on a globe button <button type="button" class="btn btn-default btn-xs">
                    <span class="glyphicon glyphicon-globe"/>
                  </button> at the start of a line will display an English translation of
                    that line</li>
                  <li>hovering your pointer over a <span class="text-success">bold type</span> Arabic term transliterated in the English text will display an English translation of that term from the Glossary</li>
                </ul>
                <h3>Critical Apparatus</h3>
                <ul>
                  <li><p>hovering your pointer over <span class="text-success">bold type</span> text in the Arabic edition will display the critical apparatus attached to that text</p>
                    <ul>
                      <li>variants are simply listed with witness identifier</li>
                      <li>text highlighted in <ins>green</ins> has been added by a later collator to fill an omission</li>
                      <li>text highlighted in <del>red</del> has been crossed out by a later collator to correct an
                        error</li>
                      <li>for substitutions, text highlighted in <ins>green</ins> is understood to replace text highlighted
                        in <del>red</del></li>
                      <li>text rubricated in the apparatus is as it appears in the associated witness</li>
                    </ul></li>
                  <li>clicking on a plus-sign button <button type="button" class="btn btn-default btn-xs">
                    <span class="glyphicon glyphicon-plus"/>
                  </button> will display any further scribal additions</li>
                  <li><p>rubricated main text is as it appears in witness (أ)</p></li>
                </ul>
                <h3>Glosses</h3>
                <ul>
                  <li>clicking on a dialogue button <button type="button" class="btn btn-default btn-xs">
                    <span class="glyphicon glyphicon-comment"/>
                  </button> will display an attached gloss</li>
                </ul>
                <h3>Folio Images (only manuscript [أ] at this time)</h3>
                <ul>
                  <li>clicking on a folio number button <button type="button" class="btn btn-default btn-xs">
                    <span class="glyphicon glyphicon-bookmark"/>
                  </button> will display an image of that entire folio</li>
                  <li>clicking on the image will open a new window in which the image is magnified; clicking on
                    the new image will magnify it further</li>
                </ul>
                <h3>Bibliography</h3>
                <ul>
                  <li>clicking on a <a>short reference</a> in the footnotes will take you to its full entry in the Works Cited. Clicking on the return button <a class="btn btn-default btn-xs"><span class="glyphicon glyphicon-menu-up" aria-hidden="true"></span></a> at the end of the full reference will take you back to the footnote</li>
                </ul>
              </div>
            </div>
          </div>
        </aside>
        <main>
          <xsl:if test="tei:text/@xml:lang">
            <xsl:attribute name="lang">
              <xsl:value-of select="tei:text/@xml:lang"/>
            </xsl:attribute>
          </xsl:if>
          <xsl:if test="tei:text/@xml:lang = exsl:node-set($rtl-languages)/*">
            <xsl:attribute name="dir">rtl</xsl:attribute>
          </xsl:if>
          <xsl:apply-templates select="tei:text"/>
        </main>
        <footer>
          <xsl:variable name="publisher" select="tei:teiHeader/tei:fileDesc/tei:publicationStmt/tei:publisher" />
          <xsl:variable name="pubPlace" select="tei:teiHeader/tei:fileDesc/tei:publicationStmt/tei:pubPlace" />
          <xsl:if test="$publisher">
            <p><small>
              <xsl:text>Published by </xsl:text>
              <xsl:choose>
                <xsl:when test="$publisher/tei:ptr">
                  <a href="{$publisher/tei:ptr/@target}" target="_blank">
                    <xsl:value-of select="normalize-space($publisher)"/>
                  </a>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="normalize-space($publisher)"/>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:if test="$pubPlace">
                <xsl:text>, </xsl:text>
                <xsl:value-of select="$pubPlace"/>
              </xsl:if>
            </small></p>
          </xsl:if>
          <xsl:variable name="license" select="tei:teiHeader/tei:fileDesc/tei:publicationStmt/tei:availability/tei:licence" />
          <xsl:if test="$license">
            <p><small>
              <xsl:choose>
                <xsl:when test="$license/@target">
                  <a rel="license" href="{$license/@target}" target="_blank">
                    <xsl:value-of select="$license" />
                  </a>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$license" />
                </xsl:otherwise>
              </xsl:choose>
            </small></p>
          </xsl:if>
        </footer>
        <!-- Modal -->
        <xsl:apply-templates select="$translation-doc//tei:ab" mode="modal"/>
        <xsl:apply-templates select="//tei:pb[@facs]" mode="modal"/>
        <!-- jQuery JavaScript -->
        <script src="js/jquery-1.12.4.min.js"/>
        <!-- Bootstrap JavaScript -->
        <script src="js/bootstrap.min.js"/>
        <script>
        $(function () {
          $('[data-toggle="popover"]').popover({html:true});
          $('.refbacklink').hide();
        })
        </script>
        <script>
          // Fix duplicate popovers for app-in-app
          $('.app-in-lem').on('show.bs.popover', function (e) {
            // Using setTimeout to make sure the popup is actually instantiated
            setTimeout(function() {
              $(e.target).parent().popover('hide')
            }, 10)
          });
          $('.app-in-lem').on('hide.bs.popover', function (e) {
            setTimeout(function() {
              $(e.target).parent().popover('show')
            }, 10)
          });
          // Hide back links when clicked
          $('.refbacklink').on('click', function (e) {
            $(e.target).closest('a').hide();
          });
          // Show bibliography when following reference links
          $('.reflink').on('click', function (e) {
            var reflink = $(e.target).closest('a');
            // Show back link
            var linkid = reflink.attr('id')
            $('#' + linkid + '-back').show();
            // Show bibliography when following reference links
            var bibentry = reflink.attr('href');
            if ($('#panel-references').hasClass('collapse in')) {
              // Already expanded
              return true;
             } else {
              // Otherwise expand first, then navigate to hash anchor
              $('#panel-references').on('shown.bs.collapse', function (e) {
                location.hash = bibentry;
                $('#panel-references').off('shown.bs.collapse');
              });
              $('#panel-references').collapse('show');
              return false;
            }
          });
        </script>
      </body>
    </html>
  </xsl:template>

  <!-- Main text -->

  <!-- Alternate block elements -->

  <xsl:template match="tei:rdg/tei:div|tei:rdg/tei:titlePage">
    <div class="panel panel-page">
      <div dir="ltr" class="panel-heading">
        <xsl:variable name="wit" select="substring-after(parent::tei:rdg/@wit, '#')"/>
        <xsl:value-of select="key('wit', $wit)/@n"/>
        <xsl:text>: </xsl:text>
        <xsl:choose>
          <xsl:when test="local-name() = 'titlePage'">
            <xsl:text>Title page</xsl:text>
          </xsl:when>
          <xsl:when test="@type = 'colophon'">
            <xsl:text>Colophon</xsl:text>
          </xsl:when>
        </xsl:choose>
      </div>
      <div class="panel-body">
        <table class="table noborder">
          <xsl:apply-templates/>
        </table>
      </div>
    </div>
  </xsl:template>

  <!-- Standard elements -->

  <xsl:template match="tei:titlePart[not(@type) or @type = 'main']">
    <xsl:variable name="trans-target" select="concat($filename, '#', @xml:id)"/>
    <tr>
      <td>
        <xsl:for-each select="$translation-doc">
          <xsl:apply-templates select="key('trans', $trans-target)" mode="button"/>
        </xsl:for-each>
      </td>
      <td>
        <h1>
          <xsl:apply-templates/>
        </h1>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="tei:byline">
    <xsl:variable name="trans-target" select="concat($filename, '#', @xml:id)"/>
    <tr>
      <td>
        <xsl:for-each select="$translation-doc">
          <xsl:apply-templates select="key('trans', $trans-target)" mode="button"/>
        </xsl:for-each>
      </td>
      <td>
        <xsl:apply-templates/>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="tei:p">
    <xsl:variable name="trans-target" select="concat($filename, '#', @xml:id)"/>
    <p>
      <xsl:apply-templates/>
    </p>
  </xsl:template>

  <xsl:template match="tei:listBibl">
    <div class="references">
      <xsl:apply-templates />
    </div>
  </xsl:template>

  <xsl:template match="tei:listBibl/tei:bibl">
    <p class="bibentry">
      <xsl:if test="@xml:id">
        <xsl:attribute name="id"><xsl:value-of select="@xml:id"/></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates />
      <xsl:for-each select="descendant-or-self::tei:bibl[@xml:id]">
        <xsl:for-each select="key('ref', concat('#', @xml:id))">
          <a class="refbacklink btn btn-default btn-xs" href="#{@xml:id}">
            <xsl:attribute name="id">
              <xsl:value-of select="@xml:id"/>
              <xsl:text>-back</xsl:text>
            </xsl:attribute>
            <span class="glyphicon glyphicon-menu-up" aria-hidden="true"></span>
          </a>
        </xsl:for-each>
      </xsl:for-each>
    </p>
  </xsl:template>

  <xsl:template match="tei:bibl[@corresp]">
    <xsl:variable name="orig_entry" select="key('bib', substring-after(@corresp, '#'))"/>
    <!-- Since we have nested bibl entries, we need to link to the outer most entry,
         which is the one that is in the bibliography.
    -->
    <xsl:variable name="outer_entry" select="$orig_entry/ancestor-or-self::tei:bibl[last()]"/>
    <a href="#{$outer_entry/@xml:id}" class="reflink">
      <xsl:if test="@xml:id">
        <xsl:attribute name="id"><xsl:value-of select="@xml:id"/></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates />
    </a>
  </xsl:template>

  <xsl:template match="tei:bibl/tei:title">
    <xsl:variable name="orig_entry" select="key('bib', substring-after(parent::tei:bibl/@corresp, '#'))"/>
    <xsl:choose>
      <xsl:when test="not(@type = 'desc')
                      and $orig_entry[@type = exsl:node-set($paper-types)/*]
                      or (parent::tei:bibl[@type = exsl:node-set($paper-types)/*]
                          and @level = 'a')">
        <q class="bibtitle">
          <xsl:apply-templates />
        </q>
      </xsl:when>
      <xsl:when test="not(level = 's' or @type = 'desc')">
        <i class="bibtitle">
          <xsl:apply-templates />
        </i>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="tei:foreign|tei:term">
    <i>
      <xsl:attribute name="class">
        <xsl:value-of select="local-name()"/>
      </xsl:attribute>
      <xsl:if test="@xml:lang">
        <xsl:attribute name="lang">
          <xsl:value-of select="@xml:lang"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </i>
  </xsl:template>

  <xsl:template match="tei:term[substring-before(@ref, '#')]">
    <!-- Add a popup for terms that link to the central glossary.
         For simplicity, we just test if something comes before the #
         to distinguish between terms accompanied by their gloss in the
         text and those that link to the central glossary.
    -->
    <xsl:variable name="gloss_id" select="substring-after(@ref, '#')"/>
    <xsl:variable name="content">
      <xsl:for-each select="$glossary-doc">
        <xsl:apply-templates select="key('term', $gloss_id)" />
      </xsl:for-each>
    </xsl:variable>
    <xsl:variable name="text">
      <i class="term">
        <xsl:apply-templates />
      </i>
    </xsl:variable>
    <xsl:call-template name="popover">
      <xsl:with-param name="title">Glossary</xsl:with-param>
      <xsl:with-param name="content">
        <xsl:apply-templates select="exsl:node-set($content)" mode="serialize" />
      </xsl:with-param>
      <xsl:with-param name="text" select="$text" />
    </xsl:call-template>
  </xsl:template>
  
  <xsl:template match="tei:metamark">
    <span class="metamark">
      <xsl:choose>
        <xsl:when test="@function = 'signe-de-renvoi'">
          <i>signe de renvoi</i>
        </xsl:when>
        <xsl:when test="@function = 'variant'">variant sign </xsl:when>
        <xsl:when test="@function = 'correction'">correction sign </xsl:when>
      </xsl:choose>
      <xsl:if test="@rend">
        <xsl:text> </xsl:text>
        <xsl:choose>
          <xsl:when test="@rend = 'curve'">in curve shape</xsl:when>
          <xsl:when test="@rend = 'ṭa'">shaped ط</xsl:when>
          <xsl:when test="@rend = 'ḥa'">shaped ح</xsl:when>
        </xsl:choose>
      </xsl:if>
      <xsl:if test="node()">
        <span>
          <xsl:apply-templates mode="hand" select="." />
          <xsl:apply-templates />
        </span>
      </xsl:if>
    </span>
  </xsl:template>

  <xsl:template match="tei:rdg/tei:note">
    <xsl:variable name="title">
      <xsl:choose>
        <xsl:when test="@type = 'gloss'">Gloss</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@type"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="parent::tei:rdg/@wit">
        <xsl:text> in </xsl:text>
        <xsl:variable name="wit" select="substring-after(parent::tei:rdg/@wit, '#')"/>
        <xsl:for-each select="$root">
          <xsl:value-of select="key('wit', $wit)/@n"/>
        </xsl:for-each>
      </xsl:if>
      <xsl:if test="@place">
        <xsl:text> (</xsl:text>
        <xsl:choose>
          <xsl:when test="@place = 'margin'">in margin</xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@place"/>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>)</xsl:text>
      </xsl:if>
    </xsl:variable>
    <xsl:variable name="content">
      <div>
        <xsl:if test="ancestor-or-self::tei:*/@xml:lang">
          <xsl:attribute name="lang">
            <xsl:value-of select="(ancestor-or-self::tei:*/@xml:lang)[1]"/>
          </xsl:attribute>
        </xsl:if>
        <xsl:if test="(ancestor-or-self::tei:*/@xml:lang)[1] = exsl:node-set($rtl-languages)/*">
          <xsl:attribute name="dir">rtl</xsl:attribute>
        </xsl:if>
        <xsl:apply-templates/>
      </div>
    </xsl:variable>
    <xsl:call-template name="popup">
      <xsl:with-param name="title" select="$title" />
      <xsl:with-param name="icon">comment</xsl:with-param>
      <xsl:with-param name="content">
        <xsl:apply-templates select="exsl:node-set($content)" mode="serialize"/>
      </xsl:with-param>
      <xsl:with-param name="placement">left</xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="tei:note" mode="footnote-label">
    <xsl:choose>
      <xsl:when test="@n">
        <xsl:value-of select="@n" />
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="count(preceding::tei:note[@type = 'footnote']) + 1"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="tei:note[@type = 'footnote']">
    <xsl:variable name="note-id" select="generate-id()"/>
    <a class="footnoteref btn btn-default btn-xs" href="#fn-{$note-id}">
      <xsl:attribute name="id">fnref-<xsl:value-of select="$note-id" /></xsl:attribute>
      <xsl:apply-templates select="." mode="footnote-label" />
      <xsl:text>&#160;</xsl:text>
      <span class="glyphicon glyphicon-menu-down" aria-hidden="true"></span>
    </a>
  </xsl:template>

  <xsl:template match="tei:note[@type = 'footnote']" mode="footnote">
    <xsl:variable name="note-id" select="generate-id()"/>
    <p class="footnote">
      <xsl:attribute name="id">fn-<xsl:value-of select="$note-id" /></xsl:attribute>
      <a href="#fnref-{$note-id}" class="footnote btn btn-default btn-xs"><xsl:apply-templates select="." mode="footnote-label"/>&#160;<span class="glyphicon glyphicon-menu-up" aria-hidden="true"></span></a> <xsl:apply-templates />
    </p>
  </xsl:template>

  <xsl:template match="tei:ab" mode="button">
    <xsl:variable name="id" select="substring-after(@corresp, '#')"/>
    <button type="button" class="btn btn-default btn-xs" data-toggle="modal" data-backdrop="false"
      data-target="#trans-modal-{ translate($id, '#.', '--') }" title="Translation">
      <span class="glyphicon glyphicon-globe"/>
    </button>
  </xsl:template>

  <xsl:template match="tei:app[tei:rdg/tei:note]">
    <!-- Apply special rendering for glosses -->
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="tei:app[tei:rdg/tei:div|tei:rdg/tei:titlePage]">
    <!-- Apply special rendering for block elements -->
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="tei:app">
    <xsl:variable name="lemma" select="tei:lem"/>
    <xsl:variable name="text">
      <xsl:choose>
        <xsl:when test="string(tei:lem)">
          <xsl:apply-templates select="tei:lem"/>
        </xsl:when>
        <xsl:otherwise>
          <button type="button" class="btn btn-default btn-xs">
            <span class="glyphicon glyphicon-plus"/>
          </button>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:call-template name="popover">
      <xsl:with-param name="title">Apparatus</xsl:with-param>
      <xsl:with-param name="text" select="$text" />
      <xsl:with-param name="placement">
        <xsl:if test="(ancestor-or-self::tei:*/@xml:lang)[1] = exsl:node-set($rtl-languages)/*">left</xsl:if>
      </xsl:with-param>
      <xsl:with-param name="class">
        <xsl:if test="ancestor::tei:lem">app-in-lem</xsl:if>
      </xsl:with-param>
      <xsl:with-param name="content">
        <xsl:for-each select="tei:rdg">
          <xsl:text>&lt;p&gt;&lt;span dir="ltr" class="witlabel"&gt;</xsl:text>
          <xsl:choose>
            <xsl:when test="tei:subst"><xsl:if test="tei:subst/@status"><xsl:value-of select="tei:subst/@status"/><xsl:text> </xsl:text></xsl:if>subsitution in </xsl:when>
            <xsl:when test="not(string(.))">missing in </xsl:when>
            <xsl:when test="not(string($lemma))">added in </xsl:when>
          </xsl:choose>
          <xsl:variable name="wit" select="substring-after(@wit, '#')"/>
          <xsl:value-of select="key('wit', $wit)/@n"/>
          <xsl:if test="string(.)">
            <xsl:text>: </xsl:text>
          </xsl:if>
          <xsl:text>&lt;/span&gt;&lt;br/&gt;</xsl:text>
          <xsl:variable name="rdg-content">
            <xsl:apply-templates select="."/>
          </xsl:variable>
          <xsl:apply-templates select="exsl:node-set($rdg-content)" mode="serialize"/>
          <xsl:text>&lt;/p&gt;</xsl:text>
        </xsl:for-each>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="tei:body[descendant::tei:ab[@n]]">
    <table class="table noborder">
      <xsl:apply-templates/>
    </table>
  </xsl:template>

  <xsl:template match="tei:ab[@n] | tei:opener[@n]">
    <xsl:variable name="trans-target" select="concat($filename, '#', @xml:id)"/>
    <tr>
      <th scope="row">
        <xsl:value-of select="@n"/>
      </th>
      <td>
        <xsl:for-each select="$translation-doc">
          <xsl:apply-templates select="key('trans', $trans-target)" mode="button"/>
        </xsl:for-each>
      </td>
      <td>
        <xsl:apply-templates/>
      </td>
    </tr>
  </xsl:template>

  <xsl:template match="tei:pb">
    <button type="button" class="btn btn-default btn-xs" dir="ltr">
      <xsl:if test="@facs">
        <xsl:attribute name="data-toggle">modal</xsl:attribute>
        <xsl:attribute name="data-backdrop">false</xsl:attribute>
        <xsl:attribute name="data-target">#facs-modal-<xsl:value-of select="generate-id()" /></xsl:attribute>
      </xsl:if>
      <span dir="ltr" class="glyphicon glyphicon-bookmark"/>
      <span dir="ltr"><span dir="ltr"><xsl:value-of
            select="key('wit', substring-after(@edRef, '#'))/@n"/></span>/<xsl:value-of select="@n"
        /></span>
    </button>
  </xsl:template>

  <xsl:template match="tei:milestone">
    <span class="milestone">
      <xsl:if test="@unit">
        <xsl:attribute name="title">
          <xsl:value-of select="@unit"/>
        </xsl:attribute>
      </xsl:if>
      <span class="glyphicon glyphicon-asterisk" style="font-size: .7em"/>
    </span>
  </xsl:template>

  <xsl:template match="tei:rdg[tei:add]">
    <!-- `rdg` which directly contains add should behave similar to `subst`. -->
    <xsl:apply-templates select="*[not(tei:metamark)]" />
    <xsl:if test="tei:add[tei:metamark]">
      <br/>
      <span dir="ltr"><xsl:for-each select="tei:add[tei:metamark]"><xsl:apply-templates select="." />
      <xsl:if test="position() != last()"><br/></xsl:if></xsl:for-each></span>
    </xsl:if>
  </xsl:template>

  <xsl:template match="tei:del">
    <del>
      <xsl:if test="@rend = 'strikethrough'">
        <xsl:apply-templates mode="hand" select=".">
          <xsl:with-param name="del">true</xsl:with-param>
        </xsl:apply-templates>
      </xsl:if>
      <xsl:apply-templates />
    </del>
  </xsl:template>

  <xsl:template match="tei:add">
    <xsl:choose>
      <xsl:when test="not(tei:metamark)">
        <ins>
          <xsl:apply-templates mode="hand" select="." />
          <xsl:apply-templates/>
        </ins>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
    <span class="annotation" dir="ltr">
      <xsl:if test="@place">
        <xsl:if test="@place = 'margin'">in </xsl:if>
        <xsl:value-of select="@place"/>
      </xsl:if>
    </span>
  </xsl:template>

  <xsl:template match="tei:subst">
    <xsl:apply-templates select="tei:del|tei:add[not(tei:metamark)]" />
    <xsl:if test="tei:add[tei:metamark]">
      <br/>
      <span dir="ltr"><xsl:for-each select="tei:add[tei:metamark]"><xsl:apply-templates select="." />
      <xsl:if test="position() != last()"><br/></xsl:if></xsl:for-each></span>
    </xsl:if>
  </xsl:template>

  <xsl:template match="tei:surplus">
    <span class="surplus">
      <xsl:apply-templates/>
    </span>
    <xsl:if test="@reason">
      <xsl:text> </xsl:text>
      <span class="annotation">
        <xsl:value-of select="@reason"/>
      </span>
    </xsl:if>
  </xsl:template>

  <xsl:template match="tei:seg | tei:label">
    <span>
      <xsl:apply-templates select="." mode="hand"/>
      <xsl:apply-templates select="." mode="rend"/>
      <xsl:apply-templates/>
    </span>
  </xsl:template>

  <xsl:template match="tei:list[@type='glossary']">
    <dl>
      <xsl:apply-templates mode="dl" />
    </dl>
  </xsl:template>

  <xsl:template match="tei:term" mode="dl">
    <dt><xsl:apply-templates /></dt>
  </xsl:template>

  <xsl:template match="tei:gloss" mode="dl">
    <dd><xsl:apply-templates /></dd>
  </xsl:template>

  <xsl:template match="tei:hi">
    <i>
      <xsl:apply-templates select="." mode="rend"/>
      <xsl:apply-templates />
    </i>
  </xsl:template>

  <xsl:template match="tei:ref[@target]">
    <a href="{@target}" target="_blank">
      <xsl:apply-templates />
    </a>
  </xsl:template>

  <xsl:template match="tei:*" mode="hand">
    <xsl:param name="del">false</xsl:param>
    <xsl:variable name="hand">
      <xsl:choose>
        <xsl:when test="@hand"><xsl:value-of select="@hand"/></xsl:when>
        <xsl:when test="parent::tei:subst/@hand"><xsl:value-of select="parent::tei:subst/@hand"/></xsl:when>
        <xsl:when test="parent::tei:add/@hand"><xsl:value-of select="parent::tei:add/@hand"/></xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$hand">
      <xsl:attribute name="class">
        <xsl:if test="$del = 'true'">
          <xsl:text>strikethrough </xsl:text>
        </xsl:if>
        <xsl:variable name="hand-local" select="substring-after($hand, '#')"/>
        <xsl:value-of select="key('hand', $hand-local)/@medium"/>
        <xsl:if test="$del = 'true'">
          <xsl:text>-del</xsl:text>
        </xsl:if>
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

  <xsl:template match="tei:*" mode="rend">
    <xsl:if test="@rend">
      <xsl:attribute name="class">
        <xsl:value-of select="@rend"/>
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

</xsl:stylesheet>
